#include "parser/statement/alter_table_statement.hpp"
#include "parser/statement/copy_statement.hpp"
#include "parser/statement/create_index_statement.hpp"
#include "parser/statement/create_schema_statement.hpp"
#include "parser/statement/create_sequence_statement.hpp"
#include "parser/statement/create_table_statement.hpp"
#include "parser/statement/create_view_statement.hpp"
#include "parser/statement/deallocate_statement.hpp"
#include "parser/statement/delete_statement.hpp"
#include "parser/statement/drop_statement.hpp"
#include "parser/statement/execute_statement.hpp"
#include "parser/statement/explain_statement.hpp"
#include "parser/statement/insert_statement.hpp"
#include "parser/statement/prepare_statement.hpp"
#include "parser/statement/select_statement.hpp"
#include "parser/statement/transaction_statement.hpp"
#include "parser/statement/update_statement.hpp"
